import json

from admin_apis import load_admin_routes
from client_apis import load_client_routes
from friday import Friday, content_type, PYTHON3, POST, status_bad_request, status_ok, status_found, status_not_found
from functions import byte_to_string, get_datastore, generate_access_token, my_test_host, \
    my_test_port, get_now_timestamp

if PYTHON3:
    from html import escape
else:
    from cgi import escape

application = Friday()
load_client_routes(wsgi_application=application)
load_admin_routes(wsgi_application=application)


@application.route("/signup", methods=['POST'])
def register(request, response):
    datastore = get_datastore()
    response.content_type = content_type['json']

    try:
        data = POST(request.environ)

        if PYTHON3:

            mdict = byte_to_string(data.parameters)

            name = str(escape(mdict['name'])).strip()
            surname = str(escape(mdict['surname'])).strip()
            phone_number = str(escape(mdict['phone_number'])).strip()
            package_name = str(escape(mdict['package_name'])).strip()
            version_code = str(escape(mdict['version_code'])).strip()
            uuid = str(escape(mdict['uuid'])).strip()
            password = str(escape(mdict['pass'])).strip()
            app_name = str(escape(mdict['app_name'])).strip()

        else:

            name = str(escape(data.parameter('name')[0])).strip()
            surname = str(escape(data.parameter('surname')[0])).strip()
            phone_number = str(escape(data.parameter('phone_number')[0])).strip()
            package_name = str(escape(data.parameter('package_name')[0])).strip()
            version_code = str(escape(data.parameter('version_code')[0])).strip()
            uuid = str(escape(data.parameter('uuid')[0])).strip()
            password = str(escape(data.parameter('pass')[0])).strip()
            app_name = str(escape(data.parameter('app_name')[0])).strip()

    except Exception as e:
        print(e)
        content = {'status': status_bad_request}
        response.content = json.dumps(content)

    else:
        # check if user is already registered
        data = datastore.select_one("select * from users where phone_number = %s", (phone_number,))

        if data:
            content = {'status': status_found,
                       'message': 'Phone number is already registered.',
                       }
            response.content = json.dumps(content)
        else:
            # store app info
            query = "insert into apps (phone_number,app_name,package_name,version_code) values (%s, %s, %s, %s) "
            parameters = [phone_number, app_name, package_name, version_code]
            datastore.insert(query=query, parameters=parameters)

            # register user
            query = "insert into users (name,surname,phone_number,access_token,uuid,password, created) values (%s, %s, %s, %s, %s, %s, %s) "

            access_token = generate_access_token()

            parameters = [name, surname, phone_number, access_token, uuid, password, get_now_timestamp()]

            if datastore.insert(query=query, parameters=parameters):
                content = {
                    'status': status_ok,
                    'access_token': access_token,
                }
            else:
                content = {'status': status_bad_request, 'message': 'registration unsuccessful'}

            response.content = json.dumps(content)


@application.route("/signin", methods=['POST'])
def signin(request, response):
    datastore = get_datastore()
    response.content_type = content_type['json']

    try:
        data = POST(request.environ)

        if PYTHON3:

            mdict = byte_to_string(data.parameters)

            phone_number = str(escape(mdict['phone_number'])).strip()
            uuid = str(escape(mdict['uuid'])).strip()
            password = str(escape(mdict['pass'])).strip()

        else:

            phone_number = str(escape(data.parameter('phone_number')[0])).strip()
            uuid = str(escape(data.parameter('uuid')[0])).strip()
            password = str(escape(data.parameter('pass')[0])).strip()

    except Exception as e:
        print(e)
        content = {'status': status_bad_request}
        response.content = json.dumps(content)

    else:
        # check if user is already registered
        data = datastore.select_one(
            "select * from users where phone_number = %s and password = %s ",
            (phone_number, 	password))

        if data:
            # check account status

            content = {'status': status_ok,
                       'access_token': data['access_token'],
                       'name': data['name'],
                       'surname': data['surname'],
                       'has_sub': "false"
                       }

            sub_query = "select sku,consume_date, expiry_date from requests left join users on users.phone_number = requests.phone_number where users.uuid = %s and users.phone_number = %s and timestamp(expiry_date) > timestamp(%s)"
            subs = datastore.select_all(query=sub_query, parameters=[uuid, phone_number, get_now_timestamp()])

            if subs:
                content['has_sub'] = "true"
                content['subs'] = subs

            response.content = json.dumps(content)
        else:
            # account not found

            content = {'status': status_not_found, 'message': 'Login failed, check number or password and try again.'}

            response.content = json.dumps(content)


@application.route("/sku", methods=['POST'])
def skues(request, response):
    datastore = get_datastore()
    response.content_type = content_type['json']

    try:
        data = POST(request.environ)

        if PYTHON3:

            mdict = byte_to_string(data.parameters)

            package_name = str(escape(mdict['package_name'])).strip()
            name = str(escape(mdict['name'])).strip()

        else:

            package_name = str(escape(data.parameter('package_name')[0])).strip()
            name = str(escape(data.parameter('name')[0])).strip()

    except Exception as e:
        print(e)
        content = {'status': status_bad_request}
        response.content = json.dumps(content)

    else:
        # check if user is already registered
        data = datastore.select_one(
            "select name,title,description,price from skues where package_name = %s and name = %s ",
            (package_name, name))

        if data:
            # check account status

            content = {'status': status_ok,
                       'data': data,
                       }

            response.content = json.dumps(content)
        else:
            # account not found

            content = {'status': status_not_found}

        response.content = json.dumps(content)


if __name__ == '__main__':
    from wsgiref.simple_server import make_server

    httpd = make_server(my_test_host, my_test_port, application)
    print('Server started at ' + my_test_host + ':' + str(my_test_port))
    print("server running on http://" + my_test_host + ":" + str(my_test_port))
    httpd.serve_forever()
